"""
Instruction Residuals for LLM Continuous Pre-Training
"""

from .residuals import (
    Residuals,
)

__version__ = "0.3.1"
__author__ = "Omar Kamali"
__email__ = "residuals@omarkama.li"

__all__ = [
    "Residuals",
]
